//
// Created by Jorge Imperial-Sosa on 9/19/21.
//

#include <string>
#include "MergerTask.h"

MergerTask::MergerTask(std::string metricName, int index) {

    this->metricName = metricName;
    this->index = index;
}
